#include "QuickShutdown.h"

QuickShutdown::QuickShutdown(HINSTANCE hInstance):EventHandler(hInstance,L"QuickShutdown",true)
{

	//Prefer function initApplication for speed enhancements...
}

QuickShutdown::~QuickShutdown(void)
{
	//delete object created in this class.
}

bool QuickShutdown::loadInfo( wstring fileTest,bool passVerif )
{

	localShutdown=L"Slide to shutdown";
	localReset=L"Slide to restart";
	localSleep=L"Slide to sleep";
	langTurnOff=L"Slide to turn off";
	langBoot=L"Boot loader";
	localCancel=L"Cancel";
	langChoose=L"Choose your language";
	langSave=L"Save";
	langProg=L"Run program";


	TextFile *tf_in = new TextFile;
	int result = tf_in->Open (fileTest.c_str(), TF_READ);
	if (result < 0)
	{
		tf_in->Close();
		return false;//silently fail...
	}

	TCHAR *line_buf = NULL;

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//it must be equal to 1 !
		if(line_buf[0]!=L'1'&&!passVerif){
			tf_in->Close();
			return false;
		}
	}else{
		tf_in->Close();
		return false;
	}

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//first shutdown:
		localShutdown=line_buf;
	}else{
		tf_in->Close();
		return false;
	}

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//Reset:
		localReset=line_buf;
	}else{
		tf_in->Close();
		return false;
	}

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//sleep:
		localSleep=line_buf;
	}else{
		tf_in->Close();
		return false;
	}

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//Cancel:
		localCancel=line_buf;
	}else{
		tf_in->Close();
		return false;
	}

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//Save:
		langSave=line_buf;
	}else{
		tf_in->Close();
		return true;
	}

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//Choose the language:
		langChoose=line_buf;
	}else{
		tf_in->Close();
		return true;
	}

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//turn off the screen:
		langTurnOff=line_buf;
	}else{
		tf_in->Close();
		return true;
	}

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//Boot loader:
		langBoot=line_buf;
	}else{
		tf_in->Close();
		return true;
	}

	result = tf_in->ReadLine (NULL, &line_buf);
	if ( result>=0 )
	{
		//Run program:
		langProg=line_buf;
	}else{
		tf_in->Close();
		return true;
	}

	tf_in->Close ();
	return true;
}

void QuickShutdown::loadStrings(){
	//we use the registry to know what language we use:
	wstring regKey=L"SOFTWARE\\Manila_QuickShutdown";

	HKEY myKey;
	DWORD disposition;
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, regKey.c_str(), 0, 0, REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS, 0, &myKey, &disposition);

	wstring langFile=L"";

	WCHAR myFile[MAX_PATH];
	DWORD sizeValue=MAX_PATH*sizeof(WCHAR);
	if(RegQueryValueEx(myKey,L"langFile", 0, 0, (LPBYTE)myFile, &sizeValue)==ERROR_SUCCESS){
		langFile=myFile;

		if(loadInfo(langFile,true)){
			RegCloseKey(myKey);
			return;
		}
		langFile=L"";
	}

	//find the good file :
	HANDLE hfind;
	WIN32_FIND_DATA wfd;
	// Initialise la recherche
	GlobalInformations* gi=GlobalInformations::getInstance();
	wstring langFiles=gi->userDirectory+L"resources\\lang\\*.txt\0";
	wstring langDir=gi->userDirectory+L"resources\\lang\\\0";
	wstring fileTmp;
	hfind = FindFirstFile(langFiles.c_str(), &wfd);
	bool fileFound=false;
	if (hfind != INVALID_HANDLE_VALUE)
	{
		do
		{
			fileTmp=wfd.cFileName;
			if(loadInfo(langDir+fileTmp)){
				fileFound=true;
				RegSetValueEx(myKey, L"langFile", 0, REG_SZ, (BYTE*)fileTmp.c_str(), fileTmp.length()*sizeof(WCHAR));
			}
		}while (!fileFound&&FindNextFile(hfind, &wfd));
		// Recherche termine, ferme la recherche
		FindClose(hfind);
		// C'est termin!!!
	}
	RegCloseKey(myKey);
}

//function to init application (add item to our window, init var, etc.)
void QuickShutdown::initApplication(){
	loadStrings();//load localized string...
	GlobalInformations* gi=GlobalInformations::getInstance();

	//First of all, we will create a Section and add it to our window!
	//A Section is a component which can be scrolled and contain other items.
	Section *sec=new Section(100,		//id of this item. Used to find item using Element::getElementByID and in function messageTreatment
		0,							//x position of top left corner (margin-left)
		0,							//y position of top left corner (margin-top)
		gi->rs.right,	//width of component. Special value : width of screen.
		L"Quick Shutdown...");			//Title of section.

	sec->scrollingAbility(false);

	int paddingHeight=100;
	if(gi->rs.bottom==480){//screen small, padding smaller:
		paddingHeight=90;
	}
	if(gi->rs.bottom==320){//screen too small, padding smaller:
		paddingHeight=75;
	}

	//now add this section to window :
	gi->window->add(sec);	//Need a pointer of type Element*. Here sec is a pointer of type Section, which derives from Element. So it works 

	MyPanel* panel;
	MyLabel* label;
	MySlider *mSlide;
	//we use the registry to know what language we use:
	wstring regKey=L"SOFTWARE\\Manila_QuickShutdown";
	HKEY myKey;
	DWORD disposition;
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, regKey.c_str(), 0, 0, REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS, 0, &myKey, &disposition);
	unsigned char val=0;
	DWORD sizeValue=sizeof(unsigned char);

	val=0;
	RegQueryValueEx(myKey,L"ShutdownActiv", 0, 0, (LPBYTE)&val, &sizeValue);
	if(val!=0){
		panel=new MyPanel(-1,0,50,gi->rs.right,L"",false,false);
		mSlide=new MySlider(1,40,0,400,true,200.0);
		panel->add(mSlide,paddingHeight);
		label=new MyLabel(0,160,0,localShutdown,0,0xFFAAAAAA,false,RGB16(3,6,3));
		label->centerV(0,77*gi->percentOfVGA);
		label->centerH(160*gi->percentOfVGA,400*gi->percentOfVGA);
		panel->add(label,paddingHeight,0);
		sec->add(panel);
	}

	val=0;
	RegQueryValueEx(myKey, L"RestartActiv", 0, 0, (LPBYTE)&val, &sizeValue);
	if(val!=0){
		panel=new MyPanel(-1,0,0,gi->rs.right,L"",false,false);
		mSlide=new MySlider(11,40,0,400,true,200.0);
		panel->add(mSlide,paddingHeight);
		label=new MyLabel(10,160,0,localReset,0,0xFFAAAAAA,false,RGB16(3,6,3));
		label->centerV(0,77*gi->percentOfVGA);
		label->centerH(160*gi->percentOfVGA,400*gi->percentOfVGA);
		panel->add(label,paddingHeight,0);
		sec->add(panel);
	}
	val=0;
	RegQueryValueEx(myKey, L"SleepActiv", 0, 0, (LPBYTE)&val, &sizeValue);
	if(val!=0){
		panel=new MyPanel(-1,0,0,gi->rs.right,L"",false,false);
		mSlide=new MySlider(21,40,0,400,true,200.0);
		panel->add(mSlide,paddingHeight);
		label=new MyLabel(20,160,0,localSleep,0,0xFFAAAAAA,false,RGB16(3,6,3));
		label->centerV(0,77*gi->percentOfVGA);
		label->centerH(160*gi->percentOfVGA,400*gi->percentOfVGA);
		panel->add(label,paddingHeight,0);
		sec->add(panel);
	}
	val=0;
	RegQueryValueEx(myKey, L"ScreenOffActiv", 0, 0, (LPBYTE)&val, &sizeValue);
	if(val!=0){
		panel=new MyPanel(-1,0,0,gi->rs.right,L"",false,false);
		mSlide=new MySlider(31,40,0,400,true,200.0);
		panel->add(mSlide,paddingHeight);
		label=new MyLabel(30,160,0,langTurnOff,0,0xFFAAAAAA,false,RGB16(3,6,3));
		label->centerV(0,77*gi->percentOfVGA);
		label->centerH(160*gi->percentOfVGA,400*gi->percentOfVGA);
		panel->add(label,paddingHeight,0);
		sec->add(panel);
	}
	val=0;
	RegQueryValueEx(myKey, L"BootActiv", 0, 0, (LPBYTE)&val, &sizeValue);
	if(val!=0){
		panel=new MyPanel(-1,0,0,gi->rs.right,L"",false,false);
		mSlide=new MySlider(41,40,0,400,true,200.0);
		panel->add(mSlide,paddingHeight);
		label=new MyLabel(40,160,0,langBoot,0,0xFFAAAAAA,false,RGB16(3,6,3));
		label->centerV(0,77*gi->percentOfVGA);
		label->centerH(160*gi->percentOfVGA,400*gi->percentOfVGA);
		panel->add(label,paddingHeight,0);
		sec->add(panel);
	}

	val=0;
	RegQueryValueEx(myKey, L"ProgActiv", 0, 0, (LPBYTE)&val, &sizeValue);
	if(val!=0){
		panel=new MyPanel(-1,0,0,gi->rs.right,L"",false,false);
		mSlide=new MySlider(51,40,0,400,true,200.0);
		panel->add(mSlide,paddingHeight);
		label=new MyLabel(50,160,0,langProg,0,0xFFAAAAAA,false,RGB16(3,6,3));
		label->centerV(0,77*gi->percentOfVGA);
		label->centerH(160*gi->percentOfVGA,400*gi->percentOfVGA);
		panel->add(label,paddingHeight,0);
		sec->add(panel);
	}

	panel=new MyPanel(-1,0,0,gi->rs.right,L"",false,false);
	MyItem* btn=new MyItem(100,40,paddingHeight/3+5,ITEM_IS_BUTTON_NORMAL,ITEM_STATE_EBL_EMPTY,L"");
	btn->hasBackgroundTransparent(true);
	panel->add(btn);
	btn=new MyItem(200,140,paddingHeight/3,ITEM_IS_BUTTON_BIG,ITEM_STATE_EBL_EMPTY,localCancel);
	btn->changeColorTxt(0x00DDDDDD,RGB16(29,29,14),ITEM_COLOR_NORMAL);
	btn->changeColorTxt(0x00DDDDDD,RGB16(29,29,14),ITEM_COLOR_HOVER);
	panel->add(btn,-1,0);//add with the 4th element
	sec->add(panel);
	RegCloseKey(myKey);
}

void QuickShutdown::extractParams(wstring &lnkTarget,wstring &parameters){
	WCHAR target[255];
	if(lnkTarget.length()==0){
		parameters=L"";
		return;
	}

	while(lnkTarget.length()>0&&lnkTarget[lnkTarget.length()-1]<=(WCHAR)20)//bad char elimiation
		lnkTarget=lnkTarget.substr(0,lnkTarget.length()-1);

	if(lnkTarget[0]==L':'){//special character
		int pos=lnkTarget.find(L" ");

		if(pos!=wstring::npos&&pos+2<lnkTarget.length()){
			parameters=lnkTarget.substr(pos+1)+parameters;
		}
		lnkTarget=lnkTarget.substr(0,pos);
		//use register to load the menu (fastest way)
		wstring whichKey=L"Software\\Microsoft\\Shell\\Rai\\";
		whichKey=whichKey+lnkTarget;
		DWORD disposition;
		HKEY regKey;
		RegCreateKeyEx(HKEY_LOCAL_MACHINE, whichKey.c_str(), 0, 0, REG_OPTION_NON_VOLATILE,
			KEY_ALL_ACCESS, 0, &regKey, &disposition);
		WCHAR iconPathWCHAR[MAX_PATH];
		DWORD sizeValue=MAX_PATH*sizeof(WCHAR);
		if(RegQueryValueEx(regKey,L"1", 0, 0, (LPBYTE)iconPathWCHAR, &sizeValue)==ERROR_SUCCESS){
			lnkTarget=iconPathWCHAR;
			extractParams(lnkTarget,parameters);
		}
	}
	//the first caracter must be \:
	if(lnkTarget[0]!=L'\\'){
		wstring win=L"\\windows\\";
		lnkTarget=win+lnkTarget;
	}
	//last: separation of parameters:
	int posExe=lnkTarget.find(L".exe");
	if(posExe!=wstring::npos){
		if(posExe+5<lnkTarget.length()){
			parameters=lnkTarget.substr(posExe+5)+parameters;
			lnkTarget=lnkTarget.substr(0,posExe+4);
		}
	}
}

//function which is called when event appends
void QuickShutdown::messageTreatment(Element* src,int idEvent,void* lParam){
	GlobalInformations* gi=GlobalInformations::getInstance();

	switch(idEvent){
	case EVENT_CLICK:
		{
			//switch using id of element :
			switch (src->getId())
			{
			case 100:{//it's the config button...
				PostMessage(gi->m_hWnd,WM_QUIT,(WPARAM)1,(LPARAM)NULL);
				SHELLEXECUTEINFO sei;
				memset(&sei, 0, sizeof(sei));
				sei.cbSize = sizeof(sei);//size of the structure in bytes
				sei.fMask=NULL;//Mask Flags-> don't close QuickShutdown
				sei.hwnd= NULL;//Handle to any message box,when this function execute
				sei.lpVerb=TEXT("open");//verb variables for open the file mentioned in lpFile
				sei.lpDirectory=NULL;//Working directory name.NULL tells the current working Dir
				sei.lpFile=((wstring)(gi->userDirectory+L"ConfigQS.exe")).c_str();//executable file Name(.exe)
				sei.lpParameters = NULL;//no parameters to file
				sei.nShow=SW_SHOWNORMAL; //It shows the window as normal windowL
				sei.hInstApp=NULL;
				//On lance le ShellExecute
				if(!ShellExecuteEx(&sei)){
					PostMessage(gi->m_hWnd,WM_QUIT,(WPARAM)1,(LPARAM)NULL);
				};
				break;
					}
			case 200://it's the button to close...
				PostMessage(gi->m_hWnd,WM_QUIT,(WPARAM)1,(LPARAM)NULL);
				break;
			}
		}//case EVENT_CLICK
	case EVENT_VALUE_CHANGED:
		{
			//switch using id of element :
			switch (src->getId())
			{
			case 1:
				{
					//it's the shutdown slider:
					int newValue=((MySlider*)src)->getValue();
					if(newValue>=99){
						//notify taskbar:
						HWND taskbarUP = FindWindow(L"HHTaskBar", NULL);
						//Post the WM_POWERBROADCAST message with the flag PBT_APMSUSPEND
						SendMessage(taskbarUP,0x218,0x4,NULL);
						Sleep(500);

						HMODULE modDLL=LoadLibrary(L"\\Windows\\HTCUtil.dll");
						if(modDLL!=NULL){
							FARPROC HTCUtilDeviceOff;
							HTCUtilDeviceOff = GetProcAddress(modDLL,L"HTCUtilDeviceOff");
							if(NULL != HTCUtilDeviceOff){
								HTCUtilDeviceOff();
							}
						}

						ExitWindowsEx(EWX_POWEROFF,0);
						SetSystemPowerState(NULL, POWER_STATE_OFF, NULL);
					}
					if(newValue<10)
						((MyLabel*)Element::getElementByID(0,TYPE_LABEL))->changeVisibility(true,false);
					else
						((MyLabel*)Element::getElementByID(0,TYPE_LABEL))->changeVisibility(false,false);
					break;
				}
			case 11:
				{
					//it's the restart slider:
					int newValue=((MySlider*)src)->getValue();
					if(newValue>=99){
						if(ExitWindowsEx(EWX_REBOOT,0)!=NULL)
							return;
						
						//If not reseted, try to use reset program :
						if(GetFileAttributes(L"\\windows\\reset.exe")!=INVALID_FILE_ATTRIBUTES){
							PostMessage(gi->m_hWnd,WM_QUIT,(WPARAM)1,(LPARAM)NULL);
							SHELLEXECUTEINFO sei;
							memset(&sei, 0, sizeof(sei));
							sei.cbSize = sizeof(sei);//size of the structure in bytes
							sei.fMask=NULL;//Mask Flags-> don't close QuickShutdown
							sei.hwnd= NULL;//Handle to any message box,when this function execute
							sei.lpVerb=TEXT("open");//verb variables for open the file mentioned in lpFile
							sei.lpDirectory=NULL;//Working directory name.NULL tells the current working Dir
							sei.lpFile=L"\\windows\\reset.exe";//executable file Name(.exe)
							sei.lpParameters = NULL;//no parameters to file
							sei.nShow=SW_SHOWNORMAL; //It shows the window as normal windowL
							sei.hInstApp=NULL;

							ShellExecuteEx(&sei);
						}
						//if here, not rebooted, last try:
						SetSystemPowerState(NULL, POWER_STATE_RESET, NULL);
					}
					if(newValue<10)
						((MyLabel*)Element::getElementByID(10,TYPE_LABEL))->changeVisibility(true,false);
					else
						((MyLabel*)Element::getElementByID(10,TYPE_LABEL))->changeVisibility(false,false);
					break;
				}
			case 21:
				{
					//it's the sleep slider:
					int newValue=((MySlider*)src)->getValue();
					if(newValue>=99){
						PostMessage(gi->m_hWnd,WM_QUIT,(WPARAM)1,(LPARAM)NULL);
						PowerPolicyNotify(PPN_POWERBUTTONPRESSED,NULL);
					}
					if(newValue<10)
						((MyLabel*)Element::getElementByID(20,TYPE_LABEL))->changeVisibility(true,false);
					else
						((MyLabel*)Element::getElementByID(20,TYPE_LABEL))->changeVisibility(false,false);
					break;
				}
			case 31:
				{
					//it's the screen off slider:
					int newValue=((MySlider*)src)->getValue();
					if(newValue>=99){
						PostMessage(gi->m_hWnd,WM_QUIT,(WPARAM)1,(LPARAM)NULL);
						SetSystemPowerState(NULL, POWER_STATE_IDLE,POWER_FORCE);

					}
					if(newValue<10)
						((MyLabel*)Element::getElementByID(30,TYPE_LABEL))->changeVisibility(true,false);
					else
						((MyLabel*)Element::getElementByID(30,TYPE_LABEL))->changeVisibility(false,false);
					break;
				}
			case 41:
				{
					//it's the boot slider:
					int newValue=((MySlider*)src)->getValue();
					if(newValue>=99){
						PostMessage(gi->m_hWnd,WM_QUIT,(WPARAM)1,(LPARAM)NULL);
						SHELLEXECUTEINFO sei;
						memset(&sei, 0, sizeof(sei));
						sei.cbSize = sizeof(sei);//size of the structure in bytes
						sei.fMask=NULL;//Mask Flags-> don't close QuickShutdown
						sei.hwnd= NULL;//Handle to any message box,when this function execute
						sei.lpVerb=TEXT("open");//verb variables for open the file mentioned in lpFile
						sei.lpDirectory=NULL;//Working directory name.NULL tells the current working Dir
						sei.lpFile=(gi->userDirectory+L"bootloader.exe").c_str();//executable file Name(.exe)
						sei.nShow=SW_SHOWNORMAL; //It shows the window as normal windowL
						sei.hInstApp=NULL;
						//On lance le ShellExecute
						ShellExecuteEx(&sei);
						SetSystemPowerState(NULL, POWER_STATE_BOOT, POWER_FORCE);
					}
					if(newValue<10)
						((MyLabel*)Element::getElementByID(40,TYPE_LABEL))->changeVisibility(true,false);
					else
						((MyLabel*)Element::getElementByID(40,TYPE_LABEL))->changeVisibility(false,false);
					break;
				}
			case 51:
				{
					//it's the command slider:
					int newValue=((MySlider*)src)->getValue();
					if(newValue>=99){
						PostMessage(gi->m_hWnd,WM_QUIT,(WPARAM)1,(LPARAM)NULL);
						wstring regKey=L"SOFTWARE\\Manila_QuickShutdown";

						HKEY myKey;
						DWORD disposition;
						RegCreateKeyEx(HKEY_LOCAL_MACHINE, regKey.c_str(), 0, 0, REG_OPTION_NON_VOLATILE,
							KEY_ALL_ACCESS, 0, &myKey, &disposition);
						WCHAR myProg[MAX_PATH];
						DWORD sizeValue=MAX_PATH*sizeof(WCHAR);
						RegQueryValueEx(myKey,L"ProgToRun", 0, 0, (LPBYTE)myProg, &sizeValue);
						RegCloseKey(myKey);
						wstring prog=myProg;
						wstring params=L"";
						extractParams(prog,params);
						SHELLEXECUTEINFO sei;
						memset(&sei, 0, sizeof(sei));
						sei.cbSize = sizeof(sei);//size of the structure in bytes
						sei.fMask=NULL;//Mask Flags-> don't close QuickShutdown
						sei.hwnd= NULL;//Handle to any message box,when this function execute
						sei.lpVerb=TEXT("open");//verb variables for open the file mentioned in lpFile
						sei.lpDirectory=NULL;//Working directory name.NULL tells the current working Dir
						sei.lpFile=prog.c_str();//executable file Name(.exe)
						sei.lpParameters=params.c_str();//params
						sei.nShow=SW_SHOWNORMAL; //It shows the window as normal windowL
						sei.hInstApp=NULL;
						//On lance le ShellExecute
						ShellExecuteEx(&sei);
					}
					if(newValue<10)
						((MyLabel*)Element::getElementByID(50,TYPE_LABEL))->changeVisibility(true,false);
					else
						((MyLabel*)Element::getElementByID(50,TYPE_LABEL))->changeVisibility(false,false);
					break;
				}
			}
		}
	}
}