﻿#pragma once
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

#define WINDLL
#define MSG_USER_PAINT 0x0410

//-----------------------------------------------------------------------------
// Include global files
//-----------------------------------------------------------------------------
#include <windows.h>
#include <ddraw.h>
#include <winuserm.h>
#include <sipapi.h>
#include <shellapi.h>
#include <string>
#include <sstream>
#include <map>

#include <vector>
#include <aygshell.h>
#include <imaging.h>

using namespace std;

class MouseCtrl;
class EventHandler;
class DirectDrawWarp;
class MessageBoxSense;
class Section;

#define MANILA_SDK_VERSION 1.53
#define DRAWING_GDI
//#define DRAWING_DDRAW


//Can be used to create a color value given Red, green and blue values :
#ifdef DRAWING_GDI
#define RGB555(r,g,b) (((0x001F&(WORD)r) << 10) |((0x1F&(WORD)g) << 5) |((0x1F&(WORD)b)))///< RGB565 : RRRR RGGG GGGB BBBB (31,63,31)
#define RGB16(r,g,b) (((0x001F&(WORD)r) << 11) |((0x3F&(WORD)g) << 5) |((0x1F&(WORD)b)))///< RGB565 : RRRR RGGG GGGB BBBB (31,63,31)
#define RGB565to32(r,g,b) (((0x000000FF&(DWORD)r) << 19) |((0x000000FF&(DWORD)g) << 10) |((0xFF000000|(DWORD)((uchar)b))<<3))///< RGB 565 to RGB32 (false 16Bit)
#define RGB555to32(r,g,b) (((0x000000FF&(DWORD)r) << 19) |((0x000000FF&(DWORD)g) << 11) |((0xFF000000|(DWORD)((uchar)b))<<3))///< RGB 555 to RGB32 (false 16Bit)
#define RGBA555to32(alpha,color) ((DWORD)alpha|(((DWORD)(0x7C00&color)) << 9) |(((DWORD)(0x03E0&color)) << 6) |(((DWORD)(0x001F&color)) <<3))///< RGB 555 to RGB32 with alpha (false 16Bit)
#else
#define RGB565to32(r,g,b) (((WORD)r << 11) |(0x3F&(WORD)g) << 5) |((0x1F&(WORD)b))// RGB565 : RRRR RGGG GGGB BBBB (31,63,31)
//#define RGB16(r,g,b) (((WORD)r << 11) |(((WORD)g) << 5) |(WORD)b)// RGB565 : RRRR RGGG GGGB BBBB (31,63,31)
#endif

#ifdef _DEBUG
#define MyASSERT(condition) ASSERT(condition)
#else
#define MyASSERT(condition)
#endif

///structure used to save info about surface (image, screen...)
typedef struct DIRECTDRAW_GDI
{
	IDirectDrawSurface* directDrawSurf;	//Not used!!
	HDC deviceInterface;				//Can be used to draw something
	HBITMAP bitmap;
	HGDIOBJ oldObj;
	BYTE nbBPP;
	int width,height;
	DIRECTDRAW_GDI(){directDrawSurf=NULL;deviceInterface=0;bitmap=0;nbBPP=0;};
}DIRECTDRAW_GDI;
///structure used to save info about overlay (surface printed above current drawing)
typedef struct GDI_OVERLAY
{
	IDirectDrawSurface* directDrawSurf;
	HDC deviceInterface;
	HBITMAP bitmap;
	int width,height;
	HWND windowOverlay;
	RECT positionOut;
	bool visible;
	GDI_OVERLAY(){directDrawSurf=NULL;deviceInterface=0;bitmap=0;visible=false;};
}GDI_OVERLAY;


///icon structs (msdn : http://msdn.microsoft.com/en-us/library/ms997538.aspx)
typedef struct
{
	BYTE        bWidth;          // Width, in pixels, of the image
	BYTE        bHeight;         // Height, in pixels, of the image
	BYTE        bColorCount;     // Number of colors in image (0 if >=8bpp)
	BYTE        bReserved;       // Reserved ( must be 0)
	WORD        wPlanes;         // Color Planes
	WORD        wBitCount;       // Bits per pixel
	DWORD       dwBytesInRes;    // How many bytes in this resource?
	DWORD       dwImageOffset;   // Where in the file is this image?
} ICONDIRENTRY, *LPICONDIRENTRY;

///icon structs (msdn : http://msdn.microsoft.com/en-us/library/ms997538.aspx)
typedef struct
{
	WORD           idReserved;   // Reserved (must be 0)
	WORD           idType;       // Resource Type (1 for icons)
	WORD           idCount;      // How many images?
	ICONDIRENTRY   idEntries[1]; // An entry for each image (idCount of 'em)
} ICONDIR, *LPICONDIR;

///icon structs (msdn : http://msdn.microsoft.com/en-us/library/ms997538.aspx)
typedef struct
{
	BITMAPINFOHEADER   icHeader;      // DIB header
	RGBQUAD         icColors[1];   // Color table
	BYTE            icXOR[1];      // DIB bits for XOR mask
	BYTE            icAND[1];      // DIB bits for AND mask
} ICONIMAGE, *LPICONIMAGE;

//////////////////////////////////////////////////////////////////////////
/// \brief	GlobalInformation Class (singleton!)
///			Used to save global informations like resources directory or painting buffers...
//////////////////////////////////////////////////////////////////////////

class WINDLL GlobalInformations
{
	GlobalInformations(wstring name);///<singleton ->can't be created or deleted !
	~GlobalInformations(void);

	///can't be accessed directly, use getInstance() !
	static GlobalInformations* resources;
	///Should be set using setName(wstring newName)!
	///Very important, without this, SDK will not find your resources
	wstring appName;
	///should not be used
	void releaseLoadedVar();
	///used to find where resources are.
	///Set userDirectory and return resourcesDirectory
	static wstring findUserDirectory(GlobalInformations* gi);
	///Used to set values for wstrings like bg_common_titlebar
	static void loadResources(GlobalInformations* gi,wstring myDirectory);
public:
	///used to set our app name.
	///automatically set when creating an object of type EventHandler (Your app should implement this class)
	static void setName(wstring newName);
	///Used to get a pointer on GlobalInformations...
	static GlobalInformations* getInstance();
	///Used to be sure that GlobalInformations are loaded
	static bool alreadyInitialized(){return resources!=NULL;};

	wstring *bg_common_titlebar;
	wstring *bg_common_divider_section;
	wstring *btn_common_msgbox_large;
	wstring *btn_common_msgbox_small;
	wstring *btn_common_normal;
	wstring *btn_switch_thirdState;
	wstring *btn_checkbox_thirdState;
	wstring *btn_common_switch;
	wstring *btn_common_checkbox;
	wstring *btn_common_slider;
	wstring *btn_slider;
	wstring *btn_common_input;
	wstring *btn_common_listitem;
	wstring *pivot_leftarrow;
	wstring *pivot_rightarrow;
	wstring *btn_common_combo_down;
	wstring *btn_common_listitem_bottom;
	wstring *btn_common_listitem_middle;
	wstring *bg_common_dotline;
	wstring *bg_msg_box;
	wstring *bg_dark;

	//////////////////////////////////////////////////////////////////////////
	///Various global values :
	///The windows. Use this to manually repaint your window
	//////////////////////////////////////////////////////////////////////////
	DirectDrawWarp* window;
	///set to true if you want that your app is minimized in case of being closed
	bool cantBeClosed;
	///Global instance of your application (usually not needed)
	HINSTANCE m_hInstance;
	///Global handle of your window
	HWND   m_hWnd;
	///Not useful now
	LPDIRECTDRAW *ldDDraw;
	///Primary Surface. Should call window->beginPaint() before using primaryBuffer
	DIRECTDRAW_GDI *primaryBuffer;
	///Back buffer... Same size than primaryBuffer
	DIRECTDRAW_GDI *backBuffer;
	///Used to print background...
	///You can do what you wan with this, but be sure to set window->hasBackground=true
	///if your background is not uniform (else scrolling artifacts)
	DIRECTDRAW_GDI *background;
	///You should not change this value (internally used to know if running in background or not)
	///Used to preserve battery life
	bool isActiv;
	///You should not change this value (internally used to know if app has focus)
	///Used to preserve battery life
	bool mustRender;
	///You should not change this value (internally used to now if something need to be drawn)
	bool mustRedraw;
	///Used to stop main app when a message box is displaying
	HANDLE waitForMSG;
	///Used to know the size of our screen
	double percentOfVGA;
	///Used to know if mobile is in landscape(true) or portrait (false)
	bool isLandscape;
	///Critical sections, not to use!
	CRITICAL_SECTION criticPaint;
	CRITICAL_SECTION criticAddBufPaint;
	///User directory, like "\program files\YourApp\"
	wstring userDirectory;
	///Set to true if you add a menubar to your window (window->setMenu(HMENU m))
	bool hasMenuBar;

	///Font used to print title
	HFONT hFontTitle;


	RECT rs;///< screen size
	RECT rcUpdateRegion;///<Update region (when drawing occur, only region into this area are redrawed)
	RECT rcVisibleDesktopTOP;///<Should not beeing used
	RECT rcVisibleDesktopBOTTOM;///<Should not beeing used

	DWORD lastTick1;///<can be used to measure time (using getTickCount())
	DWORD lastTick2;///<can be used to measure time (using getTickCount())
	DWORD lenght1;///<can be used to measure time (using getTickCount())
	DWORD lenght2;///<can be used to measure time (using getTickCount())
};


//////////////////////////////////////////////////////////////////////////
/// \brief	LocalisedString header
///			used to print errors messages in different language
//////////////////////////////////////////////////////////////////////////

class WINDLL LocalisedString{
public:
	//////////////////////////////////////////////////////////////////////////
	/// Localized data
	//////////////////////////////////////////////////////////////////////////
	static TCHAR*	windowCreationError;
	static TCHAR*	LoadImageError;
	static TCHAR*	SizeImageError;
	static TCHAR*	AddElemError;
	static TCHAR*	ResourceError;
	static TCHAR*	szDDrawError;
	static TCHAR*	szDDrawFailedMsg;
	static TCHAR*	szSetCooperativeFailMsg;
	static TCHAR*	szNoOverlaysMsg;
	static TCHAR*	szCreateOverlayFailMsg;
	static TCHAR*	szLoadOverlayFailMsg;
	static TCHAR*	szShowOverlayFailMsg;
	static TCHAR*	szNoFlipSurfacesMsg;
	static TCHAR*	szEnumAttachedSurfacesFailMsg;
	static TCHAR*	szCreateSurfaceFailMsg;
	static TCHAR*	szSetTimerFailMsg;
	static TCHAR*	szGetCapsFailMsg;
	static TCHAR*	szNoBackBufferMsg;
	static TCHAR*	szSurfaceLost;
	static TCHAR*	szGetDC;
	static TCHAR*	szCreateInputText;
	static TCHAR*	szShowKbrd;
	static TCHAR*	szCreateMyItem;


	///print debug information when it fail :
	static void explainFail(LPCTSTR error,LPCTSTR title,int quit=1);
	///print debug information when DirectDraw fail :
	static void explainFailDirectDraw(int hRet,wstring info);
	///print debug information when Imaging fail :
	static void explainFailIImage(HRESULT hRet,wstring info);
	///print debug information when Win API fail :
	static void explainFailWinAPI(wstring info);

};

#define TYPE_CHILD_SECTION	1
#define TYPE_EDIT_CTRL		2
#define TYPE_ITEM			3
#define TYPE_LABEL			4
#define TYPE_PANEL			5
#define TYPE_PICTURE		6
#define TYPE_SELECTOR		7
#define TYPE_SECTION		8
#define TYPE_SLIDER			9

//////////////////////////////////////////////////////////////////////////
/// \brief	class Element : provide basic functions...
///			All printable element will herite from this class
//////////////////////////////////////////////////////////////////////////

class WINDLL  Element
{
	static vector<Element*> listElements;
	static IImagingFactory *pImgFactory;
	static map<DWORD,IImagingFactory*> runningThread;
	static EventHandler* gestEvent;
protected:
	RECT position;///<the position relative to other components
	RECT positionAbsolute;///<the real position on screen (updated each frame)
	bool isActivated;///<for button, ctrl, etc...
	MouseCtrl* mouseEvent;///<can ask for changes
	int idElement;///<ID used by event handler (You can choose any number)
	int paddingSup;
	int typeElement;

	///load (or reload) surface
	virtual void loadSurf(){};
public:
	///Used when screen is rotated (all surfaces must be recomputed)
	static void reloadAllSurface();
	Element* owner;///<Element which contain this Element

	Element(int idElement,int &x,int &y,int typeElement,bool realPos=false);
	virtual ~Element(void);

	friend class MessageBoxSense;
	///forcing to redraw
	virtual void setMustRedraw();
	
//-----------------------------------------------------------------------------
// Virtual pure. Must be overriden !
//-----------------------------------------------------------------------------

	/**
	* to paint component with correct offset :
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y)=0;

	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y)=0;


//-----------------------------------------------------------------------------
// Optionnal behaviour :
//-----------------------------------------------------------------------------

	/**
	* execute action (Button, text...):
	* @param isRightClick true if event is a right click (long press)
	* @return true if Element take control of events...
	*/
	virtual bool run(bool isRightClick=false);
	///add componnent (list...)
	virtual void add(Element* e,int minHeight=-1,int pos=-1,bool mixWithOther=true);
	/**
	* Method called before each frame to take decision according to user gesture.
	* Use MouseCtrl::changeMouseAnalyse to capture user gesture.
	* @return In pixel, the scrolling in vertical direction
	*/
	virtual int mouseGesture(){return 0;};

	
//-----------------------------------------------------------------------------
// Getter and setter
//-----------------------------------------------------------------------------
	///Set to true if you don't want error messages "image not found"
	static bool noError;
	///Pointer to GlobalInformations
	GlobalInformations* gi;
	///Can be used to create an IImage using a BitmapData (use fromIImageToGDI to have a compatible surface)
	static IImage* getImgFromBuffer(BitmapData* dataBuffer);
	///Can be used to create a compatible surface
	static DIRECTDRAW_GDI *fromIImageToGDI(IImage** src,bool force32bits=false);
	///When image have alpha informations, use this function to force alpha recognition
	static DIRECTDRAW_GDI * loadImageWithAlpha(wstring name,bool rescall);
	///Load an image (BMP,JPG,PNG,GIF)
	static DIRECTDRAW_GDI * loadImage(wstring *name,bool rescall=false,bool transparent=false);
	
	/**
	* Create a binary surface using the color (second parameter) in the first parameter
	* @param hbmColour surface where a special color will be set to transparent
	* @param crTransparent color key used to find area set to transparent
	* @return binary image (background->white, foreground->black)
	*/
	static DIRECTDRAW_GDI* CreateBitmapMask(DIRECTDRAW_GDI *hbmColour, COLORREF crTransparent);
	///Can be used to create an IImage from an image (BMP,JPG,PNG,GIF) (use fromIImageToGDI to have a compatible surface)
	static IImage * loadIImage(wstring *name);
	///resize the image according to GlobalInformations::percentOfVGA
	static void resizeIImage(wstring *name);
	static RECT patchImage(DIRECTDRAW_GDI * bkg,wstring *name,int nbPatch,int posX,int posY);
	static EventHandler* getGestEvent() { return gestEvent; }
	static void setGestEvent(EventHandler* val) { gestEvent = val; }

	static IImagingFactory* getIImagingFactory();

	///get the element's width
	inline int getWidth(){return position.right-position.left;};
	///get the element's height
	inline int getHeight(){return position.bottom-position.top;};
	///get the element's top position (Be carefull : relative position)
	inline int getTop(){return position.top;};
	///get the element's left position (Be carefull : relative position)
	inline int getLeft(){return position.left;};
	///get the element's bottom position (Be carefull : relative position)
	inline int getBottom(){return position.bottom;};
	///get the element's right position (Be carefull : relative position)
	inline int getRight(){return position.right;};
	///get the element's ID (You can choose any number)
	inline int getId(){return idElement;};
	///get the element's type (TYPE_ITEM for example)
	inline int getType(){return typeElement;};

	inline void setWidth(int t){position.right=position.left+t;};
	inline void setHeight(int t){position.bottom=position.top+t;};
	inline void setTop(int t){position.top=t*gi->percentOfVGA;};
	inline void setBottom(int t){position.bottom=t;};
	inline void setRight(int t){position.right=t;};
	inline void setLeft(int t){position.left=t;};
	inline void recopiePosition(RECT &dest){dest=position;};

	static void redraw();
	
	virtual void unClick(){isActivated=false;};

	/**
	* Used to access to an element identified by the param id
	* @param typeExpected is optional (to be sure that the pointer is in the correct type)
	*/
	static Element* getElementByID(int id,int typeExpected=-1);
	///This method is called by components when they change size.
	virtual void sizeChanged(Element* src,int width,int height);
	/**
	* Used to write HBITMAP into a BMP...
	*/
	void WriteBMPFile(HBITMAP bitmap, LPTSTR filename, HDC hDC);
	///Save an HICON into image (PNG,JPG,BMP,GIF)
	static bool saveIcon(HICON myIcon,wstring fileDst);
	/**
	* Open Icon
	* @param fileDst where icon is (.exe)
	* @param link if fileDst have no icons, try to find from a .lnk
	*/
	static HICON loadIcon(wstring fileDst,wstring link);
	///Send message to you defined method (EventHandler::messageTreatment)
	static void sendMsg(int wParam,void* lParam);
	///Convert a HIB to a DIB
	static bool ConvertToDIB(HBITMAP& hBitmap);
};

//////////////////////////////////////////////////////////////////////////
/// \brief	Class MyPictureBox
///			Used to print a picture (can be stretchable)
//////////////////////////////////////////////////////////////////////////

class WINDLL  MyPictureBox :
	public Element
{
	DIRECTDRAW_GDI *pImage;
	RECT clipping;
	int numImage;
	int nbMaxImage;
	int widthImg;
	bool canScal;
	///load (or reload) surface
	virtual void loadSurf();
public:
	/**
	* With this constructor, height and width are found using the picture...
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param img path to the picture to print
	* @param nbImg optional, if the picture has different pattern (like buttons), see setNumImage(int i)
	* @param rescall optional, if true the picture is rescaled using GlobalInformations::percentOfVGA
	*/
	MyPictureBox(int idElement, int x,int y,wstring img,int nbImg=1,bool rescall=false);
	
	/**
	* With this constructor, height and width are found using the picture...
	* if the picture is already loaded, use this constructor
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param i pointer to the already loaded picture
	* @param nbImg optional, if the picture has different pattern (like buttons), see setNumImage(int i)
	*/
	MyPictureBox(int idElement, int x,int y,DIRECTDRAW_GDI *i,int nbImg=1);
	~MyPictureBox(void);

	/**
	* To paint the picture...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y);

	/**
	* To test if a position is inside the element...
	* @param i if the picture has different pattern, change index
	*/
	inline void setNumImage(int i){numImage=i<nbMaxImage?i:0;};

	/**
	* To change picture.
	* Be careful: resources are not released !
	*/
	inline void changeImg(DIRECTDRAW_GDI *surf){pImage=surf;};
	/**
	* change the print behavior...
	* @param canScal set to true if you want stretch your picture
	*/
	inline void setScallingAbility(bool canScal){this->canScal=canScal;};

};

#define LABEL_STATE_NORMAL		0
#define LABEL_STATE_HOVER		1
#define LABEL_STATE_DSBL		2
#define LABEL_STATE_SELECTED	3

#define TEXT_NORMAL 0
#define TEXT_BIG 1
#define TEXT_ITALIC 2
#define TEXT_SMALL 3

//////////////////////////////////////////////////////////////////////////
/// \brief	Class MyLabel
///			Used to print text
/// The text is first printed into a surface to speed later rendering...
/// Can use a lot of memory if your app have a lot of text...
//////////////////////////////////////////////////////////////////////////

class WINDLL  MyLabel :
	public Element
{
	HFONT myFont;///<To know which font this element use. Don't release this value (local copy of static font)

	DIRECTDRAW_GDI *render;///<to speed later rendering, text caching
	wstring myText;
	RECT clipping;
	RECT sizeSurface;
	DWORD previousColor;
	DWORD myColor;
	DWORD backColor;
	DDCOLORKEY ddck;

	int state;
	bool mustErase;
	bool isVisible;
	bool isCenterV;
	bool isCenterH;
	int lastWidth;
	int paddingTop;
	int paddingLeft;

	MyPictureBox *backGrnd;
	void initComponnents(int type,DWORD color);
	///load (or reload) surface
	virtual void loadSurf();

	void computeRealAlpha( COLORREF color ) ;///<Compute the surface of text
public:
	static HFONT fontTitle;///<Font for titles (10pt)
	static HFONT fontNormal;///<Normal Font (9pt)
	static HFONT fontItalic;///<Italic font (9pt)
	static HFONT fontSmall;///<small font (7pt)

	/**
	* Use this constructor if you know the size you want...
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param width width of text
	* @param height height of text (if text is too long, your bounding box will may be not respect this constraint)
	* @param text What you want to be printed
	* @param fontType Font type (TEXT_NORMAL, TEXT_BIG, TEXT_ITALIC or TEXT_SMALL)
	* @param centered Set to true if you want your text being horizontally centered)
	* @param color Color of your text (use RGB macro for easier use)
	*/
	MyLabel(int idElement, int x,int y,int width,int height,wstring text,int fontType=0,bool centered=true,DWORD color=0);
	/**
	* Use this constructor if your text has a background (if you have a MyPictureBox object, use the next constructor)
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param text What you want to be printed
	* @param fontType Font type (TEXT_NORMAL, TEXT_BIG, TEXT_ITALIC or TEXT_SMALL)
	* @param color Color of your text (use RGB macro for easier use)
	* @param realPos If true, text will not being re positioned when using not VGA devices (not using GlobalInformations::percentOfVGA)
	* @param backColor Depreciated, only for compatibility with previous versions
	*/
	MyLabel(int idElement, int x,int y,wstring text,int fontType=0,DWORD color=0,bool realPos=false,DWORD backColor=1);
	/**
	* Use this constructor if your text has a background (if you have a MyPictureBox object, use the next constructor)
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param text What you want to be printed
	* @param bkgHover Background image
	* @param fontType Font type (TEXT_NORMAL, TEXT_BIG, TEXT_ITALIC or TEXT_SMALL)
	* @param color Color of your text (use RGB macro for easier use)
	*/
	MyLabel(int idElement, int x,int y,wstring text,DIRECTDRAW_GDI *bkgHover,int fontType=0,DWORD color=0);
	/**
	* Use this constructor if you want a background below your text (if you have a surface, use the previous constructor)
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param text What you want to be printed
	* @param image Background image
	* @param paddingText top-left padding relative to background
	* @param fontType Font type (TEXT_NORMAL, TEXT_BIG, TEXT_ITALIC or TEXT_SMALL)
	* @param color Color of your text (use RGB macro for easier use)
	*/
	MyLabel(int idElement, int x,int y,wstring text,MyPictureBox* image,int paddingText,int fontType=0,DWORD color=0);
	~MyLabel(void);

	/**
	* To draw the text...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y);
	/**
	* To start action associated with this element
	* @param isRightClicked true if your want to start action associated with a right click
	* @return true if element has take control of future drawing (usually return false)
	*/
	bool run(bool isRightClicked=false);
	/**
	* To notify the item it is no longer active
	*/
	void unClick();

	/**
	* To draw the text with particular color...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	* @param color Use RGB macro to set value easily : RGB(255,0,0) will be red printed
	*/
	virtual void paintColor(DIRECTDRAW_GDI* lpDDraw,int x,int y,COLORREF color,COLORREF background);

	/**
	* To change the text
	* @param nT new text
	* @param redraw if true, the background will be erased (usually set to true)
	*/
	void setText(wstring nT,bool redraw=true);
	/**
	* To get the text
	* @return The value of the text
	*/
	wstring getText(){return myText;};
	
	/// Return the state of text (useful only when a background with different state are associated)...
	int getState(){return state;};
	/// To change the state of text (LABEL_STATE_NORMAL, LABEL_STATE_HOVER, LABEL_STATE_DSBL, LABEL_STATE_SELECTED)
	void changeState(int newState){state=newState;};

	bool isClickable;///<Set to true if you want that the text can be clicked
	
	///Use this function to center the text both horizontally and vertically
	void center();
	/**
	* Use this function to center the text vertically
	* @param top top of bounding box
	* @param bottom bottom of bounding box
	*/
	void centerV(int top=-1,int bottom=-1);
	/**
	* Use this function to center the text horizontally
	* @param left left of bounding box
	* @param right right of bounding box
	*/
	void centerH(int left,int right);
	/// use this function to set the padding top of text
	void setPaddingTop(int t){paddingTop=t;};
	///Use this function to set the padding left of text
	void setPaddingLeft(int l){paddingLeft=l;};

	///Use this function to get the padding top of text
	int getPaddingTop(){return paddingTop;};
	///Use this function to get the padding left of text
	int getPaddingLeft(){return paddingLeft;};

	/**
	* Use this function to set the background of text (to get a pointer of DIRECTDRAW_GDI, use Element::loadImage)
	* @param left left of bounding box
	* @param right right of bounding box
	*/
	void setBkg(DIRECTDRAW_GDI *surf);
	/**
	* Use this function to change the color of text
	* @param color Use RGB macro to set value easily : RGB(255,0,0) will be red printed
	* @param foregroundColor Should be true, not used...
	*/
	void changeColor(DWORD color,bool foregroundColor=true);

	/**
	* Use this function to change visibility of text
	* @param vis if true show the text, else hide the text
	* @param eraseBckground if true erase background before repaint the text (usually true)
	*/
	void changeVisibility(bool vis,bool eraseBckground=true);

};

//-----------------------------------------------------------------------------
// MouseCtrl header
//-----------------------------------------------------------------------------



#define NB_MOUSE_HISTORY 3
#define DBL_CLICK_LENGTH 0	//to activate dbc, set to 200
#define VEL_INERTIE 100.0


//////////////////////////////////////////////////////////////////////////
/// \brief	Class MouseCtrl
///			Used to control user gesture (scrolling, clicks...)
/// With this class, you can know if user try to click, double click, do a long press, scroll up/down or left/right
//////////////////////////////////////////////////////////////////////////
class WINDLL  MouseCtrl
{
	//Mouse interface :
	int mouseOldXPos, mouseOldYPos;
	int mouseClickXPos, mouseClickYPos;
	int mouseNewXPos, mouseNewYPos;
	int lastMouseXPos, lastMouseYPos;
	bool hasInfoAboutClick;//to know if user get the information about click
	bool rightClick;
	bool splClick;
	bool mouseDown;
	bool clickSelected;
	bool dblClick;
	bool scrolling;
	bool somethingAppend;
	bool clickPosGet;
	bool slideLeft;

	DWORD lastTickMouse;
	DWORD clickEvent;
	DWORD startScrolling;
	DWORD lastScroll;
	double userXVel[NB_MOUSE_HISTORY], userYVel[NB_MOUSE_HISTORY];
	int idPos;

	static CRITICAL_SECTION mouseUpdate; // This is the critical section object -- once initialized, it cannot be moved in memory

	Element* gestMouse;
	static GlobalInformations* gi;

	///private constructor : Singleton
	MouseCtrl(void);
	static MouseCtrl* instance;
	bool cantSlideLeft;
public:
	~MouseCtrl(void);

	/**
	* Use this method to obtain a valid pointer on MouseCtrl
	* @return valid pointer
	*/
	static MouseCtrl* getInstance();
	/**
	* This method is called before each frame to know what user has done
	* @return in pixels, the scrolling in vertical direction
	*/
	int updateElement();
	/**
	* This method is used by element when the control of interaction with user is no longer needed
	* @return pointer of element which had control
	*/
	Element* giveControlBack(){
		if (gestMouse!=NULL){
			Element* out=gestMouse;
			gestMouse->unClick();
			return out;
		};
		return NULL;
	};


	//-----------------------------------------------------------------------------
	// Setter
	//-----------------------------------------------------------------------------


	///used to emulate a right click in (x,y). A right click is a long press in rthe same position
	void rightBtnDown(int x,int y);
	///used to emulate a left button down in (x,y).
	void leftBtnDown(int x,int y);
	///used to emulate a left button up in (x,y).
	void leftBtnUp(int x,int y);
	///used to emulate a left button move in (x,y).
	void leftBtnMove(int x,int y);
	///used to emulate a left button movement @param val true if you want to emulate a button down, false else
	void setClickSelected(bool val){clickSelected=val;};
	///used to set the main proc for mouse analyze (should not be used)@return true if set is valid
	bool setMouseAnalyse(Element* e);
	/**
	* used to change the element which will analyze the user gestures
	* @param e new element which will be responsible of decision according to user gesture.
	* @return previous element in charge of decision. Be careful to set back this value when you no longer need to capture user gestures
	*/
	Element* changeMouseAnalyse(Element* e);
	///When you want to stop the scrolling effect, use this method
	void stopScroll();
	/**
	* This method is used know if user is trying to do something since the last time you use this method  (contrary to MouseCtrl::sthgHasChange)
	* @return true if user has done something...
	*/
	bool pickChange(){bool out=somethingAppend||scrolling;somethingAppend=false;return out;};



	//-----------------------------------------------------------------------------
	// Getter
	//-----------------------------------------------------------------------------
	///to get the speed of vertical scrolling
	int getMouseDY();
	///to get the speed of horizontal scrolling
	int getMouseDX();
	///to get the mean speed of horizontal scrolling
	double getMouseDXMean();
	/**
	* To know if left button of mouse is down 
	* @return true until user release the button
	*/
	bool isMouseDown();
	/**
	* To know if left button of mouse is down 
	* @return true only during a single frame
	*/
	bool isSelected();
	/**
	* Get infos about mouse
	* @return True if user try to do a scrolling left
	*/
	bool isSlidingLeft();
	/**
	* Get infos about mouse
	* @return True if user has done a left click
	*/
	bool isClicked();
	/**
	* Get infos about mouse
	* @return True if user has done a right click. (A right click is a long press in the same position)
	*/
	bool isRightClicked() const { return rightClick; };
	/**
	* Get infos about mouse
	* @return  True if user has done a double click
	*/
	bool isDblClicked();
	/**
	* Get infos about mouse
	* @return true if something has change since the last frame (contrary to MouseCtrl::pickChange)
	*/
	bool sthgHasChange();

	///get the mouse position (update x and y)
	void getMousePosition(int &x,int &y);
};


//-----------------------------------------------------------------------------
// MySlider header
//-----------------------------------------------------------------------------

#define VIS_SLIDE_NORMAL	0
#define VIS_SLIDE_DISABLED	2

//////////////////////////////////////////////////////////////////////////
/// \brief	Class MySlider
///			Used to draw an horizontal slider
/// You have two type of sliders : a common slider and a slider which return to origin when released
//////////////////////////////////////////////////////////////////////////
class MySlider :
	public Element
{
protected:

	static DIRECTDRAW_GDI *slideSurf;
	static DIRECTDRAW_GDI *btnSurf;

	RECT clipping;
	static int widthSlide;
	static int heightSlide;
	static int widthBtn;
	static int heightBtn;
	Section* container;

	int state;
	int position;
	int minVal,maxVal;
	bool sliderAnim;
	bool isSliding;
	void loadSurf();
public:
	/**
	* to create a common slider
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param width width of slider
	* @param visible true if you want to active the slider. If false, user will not be able to move the cursor
	* @param minVal minimum value of slider (when cursor is to the left)
	* @param maxVal maximum value of slider (when cursor is to the right)
	*/
	MySlider(int idElement, int x,int y,int width,bool visible,int minVal=0,int maxVal=100);
	/**
	* to create a slider which return to origin when released
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param width width of slider
	* @param visible true if you want to active the slider. If false, user will not be able to move the cursor
	* @param lengthAnim In milliseconds, the time the slider will take to go back to origin
	*/
	MySlider(int idElement, int x,int y,int width,bool visible, double lengthAnim);
	~MySlider(void);

	///To change to position of cursor @param newPos position of cursor (should be between minVal and maxVal)
	void updatePosition(int newPos=-1);

	/**
	* To draw the slider...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y);
	/**
	* To start action associated with this element
	* @param isRightClicked true if your want to start action associated with a right click
	* @return true if element has take control of future drawing (usually return false)
	*/
	bool run(bool isRightClicked=false);
	/**
	* To notify the item it is no longer active
	*/
	void unClick();

	/**
	* Method called before each frame to take decision according to user gesture.
	* Use MouseCtrl::changeMouseAnalyse to capture user gesture.
	* @return In pixel, the scrolling in vertical direction
	*/
	int mouseGesture();

	/**
	* Method used to change the visibility of slider
	* @param state on of these values : VIS_SLIDE_NORMAL or VIS_SLIDE_DISABLED
	*/
	void changeVisibility(int state){this->state=state;};
	///@return the position of cursor (between minVal and maxVal)
	int getValue(){return position;};
	static MySlider* slideRunning;///<When drawing the go back animation, used to know which slider need to repaint
	int mySpeed;///<Speed of slider when drawing animation
};



//-----------------------------------------------------------------------------
// Section header
//-----------------------------------------------------------------------------

#define PADDING_SECT_H 5
#define SCROLL_WIDTH 5

//////////////////////////////////////////////////////////////////////////
/// \brief	Class Section
///			The main panel of your application
/// When you create an element, you must add to this section to be printed to screen.
//////////////////////////////////////////////////////////////////////////
class WINDLL  Section :
	public Element
{
protected:

	wstring title;			//titre de la section...
	static DIRECTDRAW_GDI *bkgSurface;
	RECT rcTitle;
	RECT clipping;


	//-----------------------------------------------------------------------------
	// user gesture :
	//-----------------------------------------------------------------------------
	int posOverlay;
	static LPCRITICAL_SECTION criticAdd;

	vector<Element*> components;

	Element* mouseAnalysePrev;
	Element* drawingHook;
	int prevPos;
	int prevOverlay;
	static int TITLE_SECTION_HEIGHT;
	static int TITLE_SECTION_WIDTH;

	bool canScroll;


	void printScrollBar(DIRECTDRAW_GDI* lpDDraw);
	void computeBkgSurface();
	//load (or reload) surface
	virtual void loadSurf();
public:
	/**
	* to create a Section
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param width width of section (usually set to GlobalInformations::rs.right to take entire width)
	* @param title Title of section. If not wanted, don't use this param...
	*/
	Section(int idElement, int x,int y,int width,wstring title=L"");
	~Section(void);

	Element* selectedElement;///<When user has clicked on an element, it's the address of the element. NULL else
	static bool scrollVisible;///<Set to true if you want to enable the scrolling bar
	/**
	* To draw the section (will draw all components)...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y);
	/**
	* Method called before each frame to take decision according to user gesture.
	* Use MouseCtrl::changeMouseAnalyse to capture user gesture.
	* @return In pixel, the scrolling in vertical direction
	*/
	virtual int mouseGesture();

	/**
	* Use this method to add an element to the section
	* @param e Pointer of element to add
	* @param minHeight Minimum of height (if item is higher, his size will be used)
	* @param pos Position of element in the section (first element => pos==0). If pos<0, item is add at the end
	* @param mixWithOther Should be always false!!!
	*/
	void add(Element* e,int minHeight=-1,int pos=-1,bool mixWithOther=false);
	/**
	* Use this method to remove an element from the section
	* @param pos Position of element in the section (first element => pos==0)
	*/
	void remove(unsigned int pos);

	///Use this method to change the drawing responsibility
	void setDrawingHook(Element* e);

	///Use this method to move the vertical position of items
	inline void slideOverlay(int dY){posOverlay+=dY;};
	///Use this method if you want to update screen
	void updateScreen();
	///Used to hide or show the scrolling bar
	void overlayVisibility(bool visib);

	///Use this method when you want to unselect the hovered element
	inline void unselect(){selectedElement=NULL;}
	///This method is called by components when they change size. The height of section is then recomputed
	void sizeChanged(Element* src,int w,int h);

	/**
	* Use this method to block or enable the vertical scrolling
	* @param canScroll true is vertical scrolling is allowed, false else
	*/
	void scrollingAbility( bool canScroll ){this->canScroll=canScroll;}

	///Use this method to move the vertical position of items to the top
	void goToTop();

	/**
	* Use this method to verify if the scrolling is allowed
	* @param dy In pixels, wanted vertical scrolling
	* @param isDragged if true, the scrolling range is greater
	* @return In pixel, the real vertical scrolling
	*/
	int verifScrollingRange(int dy,bool isDragged);

	static GDI_OVERLAY *scrollBar;
};

//-----------------------------------------------------------------------------
// MyEditControl header
//-----------------------------------------------------------------------------

#define ID_EDITCHILD	102
#define IDM_EDITMENU	103

#define VIS_NORMAL 0
#define VIS_HOVER 1
#define VIS_DISABLED 2


using namespace std;

//////////////////////////////////////////////////////////////////////////
/// \brief	Class MyEditControl
///			An input text with an adjustable width
///  Easy get and set text. SIP (Soft Input Panel) detection: scroll Edit area when below SIP
//////////////////////////////////////////////////////////////////////////
class WINDLL  MyEditControl :
	public Element
{
	static DIRECTDRAW_GDI *inputDDraw;
	static unsigned char widthCorner;//10...
	static unsigned short int widthItem;//450...

	MyLabel* text;
	bool slideToTop;
	DWORD startAnim;
	DWORD startPos;

	int state;//0=normal,1=hover, 2=disabled

	RECT clipping;
	Section* container;//the owner of the edit control...

	void show();
	///load (or reload) surface
	virtual void loadSurf();
public:
	/**
	* to create an edit control
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param width width of edit control
	*/
	MyEditControl(int idElement, int x,int y,int width);//no need for height (based on standard input height)
	~MyEditControl(void);

	/**
	* To draw the edit control...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y);
	/**
	* To start action associated with this element (show soft input panel and wait for user input)
	* @param isRightClicked true if your want to start action associated with a right click
	* @return true if element has take control of future drawing (usually return false)
	*/
	bool run(bool isRightClicked=false);
	/**
	* To notify the item it is no longer active (hide Input Soft Panel)
	*/
	void unClick();
	/**
	* Method called before each frame to take decision according to user gesture.
	* Use MouseCtrl::changeMouseAnalyse to capture user gesture.
	* @return In pixel, the scrolling in vertical direction
	*/
	int mouseGesture();

	///Use this method to get text the user type
	wstring getText(){return text->getText();}
	///Use this method to change the text in the input box
	void setText( LPCWSTR param1 );
	///This method is called by text when it change size. Use to verify if text is bigger than edit control
	void sizeChanged(Element* src,int width,int height);
	static int EDIT_MENU_HEIGHT;///<Height of input box

	static HWND input;///<handle for text input...
	///Method to show Software Input Panel
	void showSIP();
	static MyEditControl* pEditCtrl;
};

//-----------------------------------------------------------------------------
// MyItem header
//-----------------------------------------------------------------------------

#define ITEM_IS_CHECKBOX		1
#define ITEM_IS_SWITCH			2
#define ITEM_IS_BUTTON_NORMAL	3
#define ITEM_IS_BUTTON_BIG		4
#define ITEM_IS_BUTTON_SHORT	5

#define ITEM_STATE_EBL_EMPTY		0
#define ITEM_STATE_EBL_SELECTED		1
#define ITEM_STATE_DSBL_EMPTY		2
#define ITEM_STATE_DSBL_SELECTED	3
#define ITEM_STATE_HOVER_EMPTY		4
#define ITEM_STATE_HOVER_SELECTED	5
#define ITEM_THIRD_STATE			6
#define ITEM_THIRD_STATE_DSBL		7

#define ITEM_COLOR_HOVER	0
#define ITEM_COLOR_DISABLED	1
#define ITEM_COLOR_NORMAL	2

//////////////////////////////////////////////////////////////////////////
/// \brief	Class MyItem
///			Different items : Button (3 types), checkboxes and switch (TripleState!). Event on click, visibility and state can be changed/get easily.
//////////////////////////////////////////////////////////////////////////
class WINDLL  MyItem :
	public Element
{
	static DIRECTDRAW_GDI *checkBoxThrd;
	static DIRECTDRAW_GDI *switchThrd;
	static DIRECTDRAW_GDI *checkBoxes;
	static DIRECTDRAW_GDI *switches;
	static DIRECTDRAW_GDI *button_normal;
	static DIRECTDRAW_GDI *button_big;
	static DIRECTDRAW_GDI *button_short;

	static int checkBoxesWidth;
	static int checkBoxesHeight;
	static int switchesWidth;
	static int switchesHeight;
	static int button_normalWidth;
	static int button_normalHeight;
	static int button_bigWidth;
	static int button_bigHeight;
	static int button_shortWidth;
	static int button_shortHeight;
	static int button_ckThrdWidth;
	static int button_ckThrdHeight;
	static int button_swThrdWidth;
	static int button_swThrdHeight;

	DIRECTDRAW_GDI *mySurface;
	RECT clipping;
	RECT positionElement;
	RECT positionText;
	int elementHeight;
	int elementWidth;
	unsigned char state;
	unsigned char type;

	DWORD colorHover,colorDsbl,colorNrmal;
	DWORD colorBackHover,colorBackDsbl,colorBackNrmal;

	MyLabel* text;
	bool isCentered;
	bool forceErase;
	///forcing to redraw
	void setMustRedraw();
	///load (or reload) surface
	virtual void loadSurf();
public:
	/**
	* to create an item (no need for size, only depends of type)
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param type one of these values: ITEM_IS_CHECKBOX, ITEM_IS_SWITCH, ITEM_IS_BUTTON_NORMAL, ITEM_IS_BUTTON_BIG or ITEM_IS_BUTTON_SHORT
	* @param state one of these values: ITEM_STATE_EBL_EMPTY, ITEM_STATE_EBL_SELECTED, ITEM_STATE_DSBL_EMPTY, ITEM_STATE_DSBL_SELECTED, ITEM_STATE_HOVER_EMPTY, ITEM_STATE_HOVER_SELECTED, ITEM_THIRD_STATE or ITEM_THIRD_STATE_DSBL
	* @param text text of label
	*/
	MyItem(int idElement, int x,int y,unsigned char type,unsigned char state,wstring text=L"");
	~MyItem(void);

	/**
	* To draw the item...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y);
	/**
	* To start action associated with this element
	* @param isRightClicked true if your want to start action associated with a right click
	* @return true if element has take control of future drawing (usually return false)
	*/
	bool run(bool isRightClicked=false);
	/**
	* To notify the item it is no longer active (unclick the button)
	*/
	void unClick();
	/**
	* Used to know if item is enabled or disabled
	* @return true if the item is enabled
	*/
	unsigned char isEnabled(){return state!=ITEM_STATE_DSBL_EMPTY&&state!=ITEM_STATE_DSBL_SELECTED&&state!=ITEM_THIRD_STATE_DSBL;};
	/**
	* @return true if user is trying to click on element
	*/
	unsigned char isSelected(){return state==ITEM_STATE_HOVER_SELECTED||state==ITEM_STATE_EBL_SELECTED;};

	/**
	* Use this method to change the text of item
	* @param text new text of item
	*/
	void changeText(wstring text);


	///Use this method to change the visibility of item.
	///Each call will switch visibility between enabled/disabled
	inline void switchVisibility(){
		if(state==ITEM_STATE_DSBL_EMPTY||state==ITEM_STATE_DSBL_SELECTED) state-=2;
		else if(state==ITEM_STATE_EBL_EMPTY||state==ITEM_STATE_EBL_SELECTED) state+=2;
		else if(state==ITEM_THIRD_STATE) state++;
		else if(state==ITEM_THIRD_STATE_DSBL) state--;
		else state-=2;
		setMustRedraw();
	}
	///This method is called by text when it change size. Use to verify if text is bigger than item
	void sizeChanged(Element* src,int width,int height);
	/**
	* Use this method to change the color of text into the component
	* @param color Color of your text (use RGB macro for easier use)
	* @param backColor Not used...
	* @return type one of these values : ITEM_COLOR_HOVER, ITEM_COLOR_DISABLED or ITEM_COLOR_NORMAL
	*/
	void changeColorTxt(DWORD color,DWORD backColor,int type);
	///Use this method to center the text into the item
	void centerH();
	///Use this method if background of button is transparent
	void hasBackgroundTransparent(bool val){forceErase=val;};
};


//-----------------------------------------------------------------------------
// MyPanel header
//-----------------------------------------------------------------------------

//#define TITLE_PANEL_HEIGHT 32
#define PADDING_PANEL_H 0//was set to 5 but padding should be manually set...

//////////////////////////////////////////////////////////////////////////
/// \brief	Class MyPanel
///			This element is a sub section and is used to bring together elements. This element can have a title and dotted separator between each element.
//////////////////////////////////////////////////////////////////////////
class WINDLL  MyPanel :
	public Element
{
	wstring title;				//titre du panel...
	static DIRECTDRAW_GDI *bkgSurfaceSrc;
	DIRECTDRAW_GDI *bkgSurface;
	static int bkgSurfaceSrcWidth;
	static DIRECTDRAW_GDI *separator;
	static int separatorWidth;
	static LPCRITICAL_SECTION criticAdd;

	RECT rcTitle;		//size of title...
	RECT clipping;


	//////////////////////////////////////////////////////////////////////////
	//Configuration :
	//////////////////////////////////////////////////////////////////////////

	bool hasTitle,hasSeparator;

	vector<Element*> components;
	vector<int> heightComp;
	vector<bool> isAlone;

	void computeSurface();
	//load (or reload) surface
	virtual void loadSurf();
public:
	/**
	* to create a panel. Height is computed based on content...
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param width width of section (usually set to GlobalInformations::rs.right to take entire width)
	* @param title Title of sub section
	* @param hasTitle set to true if you want to print the title
	* @param hasSeparator set to true if you want to print separator
	*/
	MyPanel(int idElement, int x,int y,int width,wstring title,bool hasTitle=true,bool hasSeparator=true);
	~MyPanel(void);

	/**
	* To draw the panel (and components)...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y);

	/**
	* Use this method to add an element to the sub section
	* @param e Pointer of element to add
	* @param minHeight Minimum of height (if item is higher, his size will be used)
	* @param pos Position of element in the section (first element => pos==0). If pos<0, item is add at the end
	* @param mixWithOther set to true if you want to mix this element with other
	*/
	void add(Element* e,int minHeight=-1,int pos=-1,bool mixWithOther=true);
	/**
	* Use this method to remove an element from the panel
	* @param pos Position of element in the section (first element => pos==0)
	* @param nb numbers of element to remove
	*/
	Element* remove(int pos,int nb=1);

	/**
	* Use this method to change the title of panel
	* @param title new title. (If you set it to L"", the title will be hidden)
	*/
	void changeTitle(wstring title);

	//////////////////////////////////////////////////////////////////////////
	// Getter :
	//////////////////////////////////////////////////////////////////////////

	///get element based on position in panel (first position == 0)
	Element* getElement(int pos){if((unsigned int)pos<components.size()&&pos>=0)return components[pos];else return NULL;};
	///Used to get the numbers of components into the sub section
	int getSize(){return components.size();};
	///This method is called by components when they change size. Used to recompute the size of panel
	void sizeChanged(Element* src,int width,int height);
	static int TITLE_PANEL_HEIGHT;///<Height of title of sub section
};

//-----------------------------------------------------------------------------
// MySelector header
//-----------------------------------------------------------------------------


#define SELECT_STATE_NORMAL		0
#define SELECT_STATE_HOVER		1
#define SELECT_STATE_DSBL		2
#define SELECT_STATE_SELECTED	3

//////////////////////////////////////////////////////////////////////////
/// \brief	Class MySelector
///			Used to draw dropdown list. Scroll screen if list is out of screen
//////////////////////////////////////////////////////////////////////////
class WINDLL  MySelector :
	public Element
{
	static DIRECTDRAW_GDI *header;
	static int elementWidth;
	static int elementHeight;
	static int elementListWidth;
	static DIRECTDRAW_GDI *listMiddle;
	static DIRECTDRAW_GDI *listBottom;

	MyPanel *values;
	MyLabel* selected;
	RECT clipping;
	int state;

	Section* container;//the owner of the select control...
	MyLabel* title;
	bool animation;
	bool animToVisibility;
	bool hasPaintingHook;
	int lastY;
	int animIndex;
	bool isOpen;
	DWORD startSlide;
	int itemChoose;
	int listDY;
	DIRECTDRAW_GDI* backgroundBuff;
	bool backgroundIsCaptured;
	//load (or reload) surface
	virtual void loadSurf();
public:
	/**
	* to create a dropdown list.
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param nameOfChoice Title printed when nothing is selected
	*/
	MySelector(int idElement, int x,int y,wstring nameOfChoice);
	~MySelector(void);

	/**
	* To draw the dropdown list...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y);
	/**
	* To start action associated with this element (show items)
	* @param isRightClicked true if your want to start action associated with a right click
	* @return true if element has take control of future drawing (usually return false)
	*/
	bool run(bool isRightClicked=false);
	/**
	* To notify the item it is no longer active (hide items)
	*/
	void unClick();
	/**
	* Method called before each frame to take decision according to user gesture.
	* Use MouseCtrl::changeMouseAnalyse to capture user gesture.
	* @return In pixel, the scrolling in vertical direction
	*/
	int mouseGesture();

	//////////////////////////////////////////////////////////////////////////
	// various behaviour :
	//////////////////////////////////////////////////////////////////////////

	///Add an item to the dropdown list
	void addItem(wstring word);
	/**
	* get the index of selection
	* @return the number of item selected. -1 if no item is selected
	*/
	int getSelection();
	///Used to change the selection
	void setSelection(int idSelect);
	///used to get a pointer of selected item
	///@return a pointer on selected item (NULL if nothing is selected)
	MyLabel* getItemSelected(){return selected;};
	/**
	* Method used to get access to an element.
	* Used like that:\code
	* MySelector *mySelect=new MySelector(...);
	* MyLabel *item=mySelect[2];//get the  second item !
	* \endcode
	* @param i indice of wanted element
	* @return pointer of i-th element (NULL if out of range)
	*/
	MyLabel* operator[] (unsigned int i);
	/**
	* When content is changed, use this function to update the title
	* If nothing is selected, the param is used to change title, else it's the selected label
	* @param wantedTitle if nothing is selected, it will be the title, else it's the selected label
	*/
	void updateTitle(wstring wantedTitle=L"");
};

//-----------------------------------------------------------------------------
// ChildSection header
//-----------------------------------------------------------------------------

#define PADDING_ARROW 15

#define STATE_NORMAL	0
#define STATE_HOVER		1
#define STATE_DSBL		2
#define STATE_SELECTED	3

//////////////////////////////////////////////////////////////////////////
/// \brief	Class ChildSection
///			Contains a subsection which is displayed with a horizontal sliding effect.
//////////////////////////////////////////////////////////////////////////
class WINDLL  ChildSection :
	public Element
{

	void slideScreen(DIRECTDRAW_GDI* lpDDraw);
protected:
	static DIRECTDRAW_GDI *backgroundLeft;//for speed, we copy arrow on background...
	static DIRECTDRAW_GDI *backgroundRight;

	static int arrowWidth;
	static int arrowHeight;
	static int backgroundHeight;
	static int backgroundWidth;

	Section* container;//the owner of the ChildSection...
	Section* ourSection;
	wstring savTitle;
	int savPaddingTopTitle;
	MyLabel* title;
	MyLabel* subTitle;


	double durationSlide;
	bool sliding;
	bool slidToLeft;
	bool isInOurSection;
	unsigned char state;
	RECT clipping;
	//load (or reload) surface
	virtual void loadSurf();
public:
	/**
	* to create a dropdown list.
	* @param idElement a number used to identify the Element (Can be used to get a pointer with Element::getElementByID)
	* @param x padding left of the element
	* @param y padding top of the element
	* @param title Title printed on the button
	* @param subtitle subTitle printed on the button (optional)
	*/
	ChildSection(int idElement,int x,int y,wstring title,wstring subtitle=L"");
	~ChildSection(void);

	/**
	* To draw the Child section (only the button)...
	* @param lpDDraw surface to print (GlobalInformations::primaryBuffer for example)
	* @param x position of top-left
	* @param y position of top-left
	*/
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	/**
	* To test if a position is inside the element...
	* @param x position of click
	* @param y position of click
	* @return NULL if not clicked, this else
	*/
	virtual Element* isClicked(int x,int y);
	/**
	* To start action associated with this element (slide to left the panel)
	* @param isRightClicked true if your want to start action associated with a right click
	* @return true if element has take control of future drawing (usually return false)
	*/
	bool run(bool isRightClicked=false);
	/**
	* To notify the item it is no longer activ
	*/
	void unClick();

	/**
	* Method called before each frame to take decision according to user gesture.
	* Use MouseCtrl::changeMouseAnalyse to capture user gesture.
	* @return In pixel, the scrolling in vertical direction
	*/
	int mouseGesture();

	/**
	* Use this method to add an element to the subSection
	* @param e Pointer of element to add
	* @param minH Minimum of height (if item is higher, his size will be used)
	* @param pos Position of element in the section (first element => pos==0). If pos<0, item is add at the end
	* @param mixWithOther Should be always false!!!
	*/
	void add(Element *e,int minH=-1,int pos=-1,bool mixWithOther=false){ourSection->add(e,minH,pos,mixWithOther);};
	/**
	* Use this method to remove an element from the subsection
	* @param pos Position of element in the section (first element => pos==0)
	*/
	void remove(int pos){ourSection->remove(pos);};

	///To change the duration of sliding effect
	///@param newLength in Millisecond, the duration of sliding effect
	void changeSpeed(double newLength=200){durationSlide=newLength;};
};


//-----------------------------------------------------------------------------
// MessageBoxSense header
//-----------------------------------------------------------------------------


#define MESSAGE_ALERT		1
#define MESSAGE_YESNO		2
#define MESSAGE_OKCANCEL	3
#define MESSAGE_ABORTRETRY	4

#define MSG_REP_NO			0
#define MSG_REP_YES			1
#define MSG_REP_CANCEL		0
#define MSG_REP_OK			1
#define MSG_REP_ABORT		0
#define MSG_REP_RETRY		1

#define MESSAGE_REP			0x0500

//////////////////////////////////////////////////////////////////////////
/// \brief	Class MessageBoxSense
///			To print a messagebox. Four type of messagebox: info, Ok/Cancel, Abort/Retry and Yes/No.
//////////////////////////////////////////////////////////////////////////
class MessageBoxSense{
	MessageBoxSense(wstring title,wstring message,int type);

	static DIRECTDRAW_GDI *msgBox;
	static RECT rectBox;
	static int answer;
	static HANDLE threadMSG;
	static bool standAlone;

	static long FAR PASCAL messageDispatch(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	long messageAnalysis(UINT message, WPARAM wParam, LPARAM lParam);

	RECT posAbsolute;
	bool isInit;
	bool isActive;
	bool isAlreadyComputed;
	Element* selectedButton;
	int type;
	int heightMsg;
	int widthTitle;
	wstring title;
	wstring message;
	MyItem** button;

	bool repaintButton;///<Used internally to know if buttons have to be repainted.

	static GlobalInformations* gi;
public:
	~MessageBoxSense();
	HWND hWnd;///<Handle of messageBox.
	/**
	* Use this method to show a message box.
	* Be careful : if used alone, set the name of your application using GlobalInformations::setName(L"MyName") BEFORE using this method !
	* @param title Title of message box
	* @param message Message printed
	* @param type one of these values : MESSAGE_ALERT, MESSAGE_YESNO, MESSAGE_OKCANCEL or MESSAGE_ABORTRETRY
	* @return one of these values : MSG_REP_NO, MSG_REP_YES, MSG_REP_CANCEL, MSG_REP_OK, MSG_REP_ABORT or MSG_REP_RETRY
	*/
	static int show(wstring title,wstring message,int type);
	/**
	* Use this method to redraw message box (used internally)
	* @param screen surface to print the message box.
	*/
	void updateFrame(DIRECTDRAW_GDI* screen);
	///load (or reload) surfaces
	virtual void loadSurf();
};


//-----------------------------------------------------------------------------
// EventHandler header
//-----------------------------------------------------------------------------

#define EVENT_CLICK			1
#define EVENT_VALUE_CHANGED 2
#define EVENT_RIGHT_CLICK	3

//////////////////////////////////////////////////////////////////////////
/// \brief	Class EventHandler
///			This class will be your command center... All messages and windows content are created here
//////////////////////////////////////////////////////////////////////////
/**
* You will have to create a class that inherits from <b>EventHandler</b>. You have to implement the two followings methods: <br /> 
* void messageTreatment(Element* src,UINT msg);<br /> 
* void initApplication();<br /> 
* <br /> 
* Your constructor should have a HINSTANCE parameter.<br /> 
* <br /> 
* So, you will have something like that:
* \code class MyApp:
* public EventHandler
* {
* protected:
* //function to init application (add item to our window, init var, etc.)
* void initApplication();
* public:
* MyApp(HINSTANCE hInstance);
* ~MyApp(void);
* //function which is called when events appends
* void messageTreatment(Element* src,UINT msg);
* };
* \endcode
*/
class WINDLL  EventHandler
{
	HANDLE hThread;
	DWORD idThread;
	static DWORD WINAPI launchApp( LPVOID lpParameter );
protected:
	///instance from winMain
	HINSTANCE gInstance;
	/**
	* This method should be used to init application (add item to our window -via GlobalInformations::window-, init var, etc.)<br /> 
	*/
	virtual void initApplication()=0;

public:
	/**
	* Constructor of EventHandler
	@param hInstance instance from winMain
	@param windowName Name of your application
	@param topmost If true your app will be in fullscreen mode (no top bar)
	@param menu If wanted, the handle of menu created with CreateMenu (see http://msdn.microsoft.com/en-us/library/aa921327.aspx)
	*/
	EventHandler(HINSTANCE hInstance,wstring windowName,bool topmost=false,HMENU menu=NULL);
	~EventHandler(void);

	/**
	* This method is called when event appends (clicks or value change). The parameter src is a pointer on the active Element and idEvent is equal to EVENT_CLICK, EVENT_VALUE_CHANGED, EVENT_RIGHT_CLICK or wParam...<br /> 
	* msg WM_USER and WM_COMMAND received by windows are send here. In this case src==NULL, idEvent=wParam and lParam=lParam
	* @param src pointer on the Element which fire this event
	* @param idEvent number to identify the event (EVENT_CLICK, EVENT_VALUE_CHANGED, EVENT_RIGHT_CLICK or wParam)
	* @param lParam NULL in most of case, except when the message is from windows (WM_USER and WM_COMMAND)
	*/
	virtual void messageTreatment(Element* src,int idEvent,void* lParam=NULL)=0;
	///not needed, used by components to send messages to our application
	void messageProc(void* src,UINT msg=EVENT_CLICK);
	///main function. Will listen for messages and dispatch them.
	void runApp();
	///Used by messageBox to access directly to message queue.
	static void msgExecute(MSG msg);
	///Use this method to show or hide your window
	///@param isVisible if true show the window, else hide the window
	void showWindow(bool isVisible);
};

//-----------------------------------------------------------------------------
// DirectDrawWarp header
//-----------------------------------------------------------------------------

#define SW_MINIMIZED	6
#define MS_PER_FRAME	25
#define NB_FRAME_ANIM	1

//////////////////////////////////////////////////////////////////////////
/// \brief	Class DirectDrawWarp
///			This class is responsible for displaying the window.
//////////////////////////////////////////////////////////////////////////
class WINDLL  DirectDrawWarp
{
private:

	//-----------------------------------------------------------------------------
	// data
	//-----------------------------------------------------------------------------

	DWORD previousFrame;
	bool isTopMost;

	static GlobalInformations* gi;

	const TCHAR* name;

	MouseCtrl* mouse;
	MessageBoxSense* messageBox;

	//-----------------------------------------------------------------------------
	// Window content
	//-----------------------------------------------------------------------------

	wstring imgBkgFile;
	wstring imgBkgFileLand;
	vector<Section*> section;
	int sectionActiv;
	int numRefreash;
	vector<RECT> bufferPaint;
	vector<bool> bufferPaintBack;

	bool isWindowInitialized;
	bool hasBackground;
	HWND g_hWndMenuBar;
	//-----------------------------------------------------------------------------
	// Fonctions
	//-----------------------------------------------------------------------------

	// Desc: The Main Window Procedure
	static long FAR PASCAL StaticWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	long WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	int renderProc();

	int UpdateFramePriv(bool eraseBack=false);

public:
	/**
	* Constructor of EventHandler
	@param hInstance instance from winMain
	@param name Name of your application
	@param topmost If true your app will be in fullscreen mode (no top bar)
	@param menu If wanted, the handle of menu created with CreateMenu (see http://msdn.microsoft.com/en-us/library/aa921327.aspx)
	*/
	DirectDrawWarp(HINSTANCE hInstance,const TCHAR* name,bool topmost=false,HMENU menuBottom=NULL);
	/**
	* Constructor which will not create a window (used by messageBox)
	*/
	DirectDrawWarp();
	~DirectDrawWarp(void);
	
	///Use this method if you want to redraw the screen
	///before calling this method, set the region GlobalInformations::rcUpdateRegion!
	///@param eraseBack if true, erase the background
	void UpdateFrame(bool eraseBack=false);
	///Use this method to update elements according to an user gesture
	void updateElements(){this->mouse->updateElement();};

	///Use this method to add a Section to the window
	void add(Section* e);

	///When a message box append, tell to the window to wait...
	void changeControl(MessageBoxSense* mb);

	///first method used to init the window (call to EventHandler::initApplication)
	void initWindow();
	///When something has change, you can ask for a redraw of a part of the screen
	///@param rc RECT of update
	///@param backErase if true, background will be erased
	void addToRedraw(RECT &rc,bool backErase);
	///method which is called when something is waiting to a redraw (via addToRedraw)
	///@param dy set to 0 usually...
	void doRepaintBuffered(int dy);
	/**
	* This function is used to draw surface on another with alpha transparency when available
	* @param out surface which will be painted
	* @param dest RECT of output (if NULL, entire surface)
	* @param srcSur surface source to paint on <i>out</i>
	* @param src RECT of input (if NULL, entire surface)
	* @param flag if DDBLT_COLORFILL, srcSur will not be used but param->dwFillColor
	* @param param if flag=DDBLT_COLORFILL, param->dwFillColor is the color to paint
	*/
	HRESULT BltFast(DIRECTDRAW_GDI* out,RECT *dest,DIRECTDRAW_GDI* srcSur, RECT *src,DWORD flag,LPDDBLTFX param);
	/**
	* This function is used to draw surface on another with a mask
	* @param out surface which will be painted
	* @param dest RECT of output (if NULL, entire surface)
	* @param srcSur surface source to paint on <i>out</i>
	* @param maskBlt binary image where white will be transparant and black will be opaque
	* @param src RECT of input and mask (if NULL, entire surface)
	*/
	HRESULT BltMask(DIRECTDRAW_GDI* out,RECT *dest,DIRECTDRAW_GDI* srcSur,DIRECTDRAW_GDI* maskBlt, RECT *src);
	/**
	* This function is used to draw surface on another with a constant alpha mix
	* @param out surface which will be painted
	* @param dest RECT of output (if NULL, entire surface)
	* @param srcSur surface source to paint on <i>out</i>
	* @param src RECT of input and mask (if NULL, entire surface)
	* @param constantAlpha if 0, totally transparent, if 1 totally opaque
	*/
	void BltMix(DIRECTDRAW_GDI* out,RECT *dest,DIRECTDRAW_GDI* srcSur, RECT *src,DWORD constantAlpha);
	/**
	* This function is used to remove from memory a surface (free the resources)
	* @param surf surface to free
	*/
	static void deleteSurf(DIRECTDRAW_GDI** surf);
	/**
	* This function is used to create a surface
	* @param desc infos for new surface (desc->dwWidth for width, desc->dwHeight for height and desc->dwBackBufferCount for number of bytes per pixels (16,24 or 32)
	* @param surf new created surface.
	*/
	static HRESULT createSurf(DDSURFACEDESC* desc,DIRECTDRAW_GDI* surf);
	/**
	* This function is used to create an overlay
	* @param desc see http://msdn.microsoft.com/en-us/library/aa911086.aspx
	*/
	static GDI_OVERLAY* createOverlay(DDSURFACEDESC* desc);
	///when drawing on screen use this function before
	static void beginPaint();
	///when drawing on screen use this function after
	static void endPaint();

	/**
	* To set an image in background
	* @param imgSrc path to image (if imgSrc=NULL, just update the background)
	* @param forLandscape indicate if imgSrc is for landscape or not
	*/
	void setBackgroundImg( LPCWSTR imgSrc=NULL,bool forLandscape=false);

	/**
	* use this method to add or remove the bottom bar
	* for more informations, see http://msdn.microsoft.com/en-us/library/aa921327.aspx
	* @param hMenu new menu, if NULL remove the menu
	*/
	void setMenu( HMENU hMenu );

	/**
	* use this method to send message to the toolbar
	* for more informations, see http://msdn.microsoft.com/en-us/library/aa930112.aspx
	* @param message for example TB_SETBUTTONINFO
	* @param idMenu id of item (user defined)
	* @param info depend of message type...
	*/
	void changeMenu( UINT message,int idMenu, LPARAM info ) ;
	/**
	* use this method to change the background color
	* @param newColor (use RGB macro for easier use)
	*/
	void setBackgroundColor( COLORREF newColor);

	static int background_color;///<background color (don't change, use setBackgroundColor instead)
	RECT hMenuArea;///<size of bottom menu bar
	HMENU myHMenu;///<handle of bottom menu bar
};
